/* Copyright (C) Perinet */
'use strict';



var iOS = /iPad|iPhone|iPod/.test( navigator.userAgent ) && !window.MSStream;

var supports_touch = 'ontouchstart' in window;

//----------------------------------------------------------------------

function s_id()
{
	return Math.random().toString( 16 ).substr( 2, 8 );
}

//----------------------------------------------------------------------

function parse_url( url )
{
	var pattern = RegExp( "^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?" );
	var matches = url.match( pattern );
	return {
		scheme: matches[2],
		authority: matches[4],
		path: matches[5],
		query: matches[7],
		fragment: matches[9]
	};
}

//----------------------------------------------------------------------

function checkauth( resp )
{
	if( resp.search( "Not authorized" ) != -1 )
	{
		var resobj = jQuery.parseJSON( resp );
		var result = resobj["error"]["data"]["message"];

		info_dialog( result );
		return false;
	}
	else
	{
		return true;
	}
}

//----------------------------------------------------------------------

function show_error( errmsg )
{
	info_dialog( "An Error occurred: <br/> " + errmsg );
}

//----------------------------------------------------------------------

function urlExists( image_url )
{
	var http = new XMLHttpRequest();

	http.open( 'HEAD', image_url, false );
	http.send();

	var s = http.status;

	return s == 200;
}

//----------------------------------------------------------------------

function urlExistsAsync( url, callback )
{
	$.ajax( {
		type: 'GET',
		url: url,
		success: function()
		{
			callback( true );
		},
		error: function()
		{
			callback( false );
		}
	} );
}

//----------------------------------------------------------------------

function valid_hostname( str )
{
	var pattern = /^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9])$/
	return pattern.test( str )
}

//----------------------------------------------------------------------

function num2dec( num )
{
	num = Math.max( Math.min( num, 32 ), 0 )
	var all = Math.pow( 2, ( num ) ) - 1
	var binstr = all.toString( 2 );
	while( binstr.length < 32 )
		binstr += "0";

	var a = parseInt( binstr.substring( 0, 8 ), 2 );
	var b = parseInt( binstr.substring( 8, 16 ), 2 );
	var c = parseInt( binstr.substring( 16, 24 ), 2 );
	var d = parseInt( binstr.substring( 24, 32 ), 2 );
	return a + "." + b + "." + c + "." + d
}

//----------------------------------------------------------------------

function dec2num( dec )
{
	var b = dec.split( "." );
	if( b.length != 4 )
		return -1;

	var all = parseInt( b[0] ) * 16777216 + parseInt( b[1] ) * 65536 + parseInt( b[2] ) * 256 + parseInt( b[3] );
	var binstr = ( all >>> 0 ).toString( 2 );

	var num = binstr.replace( /[^1]/g, '' ).length
	return num
}

//----------------------------------------------------------------------

function checkNum( ref )
{
	var v = ref.value.trim();
	if( v == "" )
		return;

	var asInt = parseInt( v, 10 );

	if( asInt <= 32 )
		ref.value = num2dec( asInt );
}

//----------------------------------------------------------------------

var events = ( function()
{
	var topics = {};
	var hOP = topics.hasOwnProperty;

	return {
		subscribe: function( topic, listener )
		{

			if( !hOP.call( topics, topic ) )
				topics[topic] = [];

			var index = topics[topic].push( listener ) - 1;

			return {
				remove: function()
				{
					delete topics[topic][index];
				}
			};
		},
		publish: function( topic, info )
		{

			if( !hOP.call( topics, topic ) )
				return;

			topics[topic].forEach( function( item )
			{
				item( info != undefined ? info : {} );
			} );
		}
	};
} )();



//----------------------------------------------------------------------

function file_download( filename )
{
	let filetoget = filename
	let service_url = window.location.protocol + "//" + window.location.host + "/file_down/"

	var iframeid = "#iframe_download_helper";
	if( $( iframeid ).length > 0 )
		$( iframeid ).remove()

	let query_str = service_url + "?what=" + filetoget

	let call_data = '<div id="iframe_download_helper" style="display:none;"><iframe src="' + query_str + '" style="border:0;width:0;height:0;"></iframe></div>';
	$( 'body' ).append( call_data );
}

//----------------------------------------------------------------------

function download( data, filename, type )
{
	var a = document.createElement( "a" ),
		file = new Blob( [data], { type: type } );
	if( window.navigator.msSaveOrOpenBlob )
		window.navigator.msSaveOrOpenBlob( file, filename );
	else
	{ // Others
		var url = URL.createObjectURL( file );
		a.href = url;
		a.download = filename;
		document.body.appendChild( a );
		a.click();
		setTimeout( function()
		{
			document.body.removeChild( a );
			window.URL.revokeObjectURL( url );
		}, 0 );
	}
}

//----------------------------------------------------------------------

function upload_file( file_obj, target_filename, progress_callbk, success_callbk, fail_callbk )
{
	var func = file_obj.mozSlice ? 'mozSlice' : ( file_obj.webkitSlice ? 'webkitSlice' : 'slice' );
	var chunk_size = 1024 * 1024;
	var total_bytes = file_obj.size;
	var f_name = target_filename;

	base.call2( "transfer_start", [f_name, total_bytes, chunk_size, h_storage.token],
		function( resp )
		{
			console.log( "success transfer start" )
			send_part( 0 );
		},
		function( err )
		{

			info_dialog( "File Upload could not be initiated.<br>(" + err + ")" );
			fail_callbk( err );
		}
	);


	function send_part( offset )
	{
		get_b64_chunk( offset, chunk_size,
			function( b64str )
			{
				if( b64str != "" )
				{
					var chunk_num = parseInt( offset / chunk_size );
					// Unfortunately this cannot be done via websockets
					base.call2( "transfer_chunk", [f_name, chunk_num, b64str, h_storage.token],
						function( resp )
						{
							var completed = resp.search( "completed" ) != -1;

							if( !completed )
							{
								progress_callbk( offset / total_bytes );
								send_part( offset + chunk_size );
							}
							else
							{
								progress_callbk( 1 );
								success_callbk( resp );
							}
						},
						function( err )
						{
							show_error( err );
							fail_callbk( err );
						}
					);
				}
			},
			function() { }
		);
	}

	function get_b64_chunk( offset, sz, success, fail )
	{
		var bytes = file_obj[func]( offset, offset + sz );
		var reader = new window.FileReader();
		reader.onloadend = function()
		{
			var result_split = reader.result.split( "base64," );
			var hasdata = result_split.length >= 2 && result_split[1] != "";
			var b64str = hasdata ? result_split[1] : "";

			success( b64str );
		}
		reader.readAsDataURL( bytes );
	}
}

//----------------------------------------------------------------------

function get_name_from_hash()
{
	var info = location.hash.split( "_" );
	var name = info[2] || "";
	return name;
}

//----------------------------------------------------------------------

function arr_2_str( arr )
{
	return String.fromCharCode.apply( null, arr );
}

function blob_2_str( bl )
{
	return String.fromCharCode.apply( null, new Uint8Array( bl ) );
}


function get_buffer_from_blob( blob_obj )
{
	var temporaryFileReader = new FileReader();
	return new Promise( ( resolve, reject ) =>
	{
		temporaryFileReader.onerror = () =>
		{
			temporaryFileReader.abort();
			return reject( "error reading input file" );
		};
		temporaryFileReader.onload = () =>
		{
			var plob = temporaryFileReader.result;
			if( plob.byteLength == 0 )
				return reject( "empty result" );

			return resolve( plob );
		};

		temporaryFileReader.readAsArrayBuffer( blob_obj );
	} );
}

//----------------------------------------------------------------------

var get_ie_version = function()
{
	var sAgent = window.navigator.userAgent;
	var Idx = sAgent.indexOf( "MSIE" );

	// If IE, return version number.
	if( Idx > 0 )
	{
		return parseInt( sAgent.substring( Idx + 5, sAgent.indexOf( ".", Idx ) ) );
	}
	// Condition Check IF IE 11 and or MS Edge
	else if( !!navigator.userAgent.match( /Trident\/7\./ ) || window.navigator.userAgent.indexOf( "Edge" ) > -1 )
	{
		return 11;
	}
	else
	{
		return 0; //It is not IE
	}
};

//----------------------------------------------------------------------

function validate_ip( addr_, mode, no_placeholder )
{
	let addr = addr_.trim();
	if( mode == 4 && addr.indexOf( ":" ) != -1 )
		return false;

	if( mode == 6 && addr.indexOf( "." ) != -1 )
		return false;

	// see http://jsfiddle.net/opd1v7au/2/
	let valid_syntax = /((^\s*((([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))\s*$)|(^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$))/.test( addr );
	if( typeof no_placeholder == 'undefined' )
		return valid_syntax;

	return mode == 4 ? valid_syntax && addr != "0.0.0.0"
		: valid_syntax && addr != "::";

}

//----------------------------------------------------------------------

function is_special_ipv4( addr_ )
{
	let addr = addr_.trim();

	let current_network = /^0?0?0\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/.test( addr );
	let loopback_addresses = /^127\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/.test( addr );
	let broadcast_address = /^255\.255\.255\.255$/.test( addr );

	return current_network || loopback_addresses || broadcast_address
}

//----------------------------------------------------------------------

function is_special_ipv6( addr_ )
{
	let addr = addr_.trim();

	let unspecified_address_or_default_route = /^(::|0:0:0:0:0:0:0:0)$/.test( addr );
	let loopback_address = /^(::1|0:0:0:0:0:0:0:1)$/.test( addr );

	return unspecified_address_or_default_route || loopback_address
}

//----------------------------------------------------------------------

async function get_containername_list()
{
	try
	{
		let cont_conf = await base.get_container( ".vmsettings" );
		let name_sorted_by_index = [];
		for( let name in cont_conf )
		{
			name_sorted_by_index[cont_conf[name].index] = name;
		}

		return name_sorted_by_index;
	}
	catch( err )
	{
		console.error( err );
		return [];
	}
}

//----------------------------------------------------------------------

function showdown_convert( input )
{
	var converter = new showdown.Converter( { extensions: ['table'] } );
	converter.setOption( 'simpleLineBreaks', true );
	return converter.makeHtml( input );

}

//----------------------------------------------------------------------

// polyfill for Object.entries
if( !Object.entries )
{
	Object.entries = function( obj )
	{
		var ownProps = Object.keys( obj ),
			i = ownProps.length,
			resArray = new Array( i ); // preallocate the Array
		while( i-- )
			resArray[i] = [ownProps[i], obj[ownProps[i]]];

		return resArray;
	};
}

//----------------------------------------------------------------------

( function()
{
	Object.map = function( obj, fn, ctx )
	{
		return Object.keys( obj ).reduce( ( a, b ) =>
		{
			a[b] = fn.call( ctx || null, b, obj[b] );
			return a;
		}, {} );
	};


	Object.filter = function( obj, predicate )
	{
		let result = {}
		let key;
		for( key in obj )
		{
			if( obj.hasOwnProperty( key ) && !predicate( obj[key] ) )
			{
				result[key] = obj[key];
			}
		}

		return result;
	};
} )();

//----------------------------------------------------------------------

function reboot()
{
	return new Promise(
		( resolve, reject ) =>
		{
			base.call2( "set_base", [".reboot"],
				suc =>
				{
					base.refresh( 10000 );
					resolve( suc );
				},
				err =>
				{
					reject( err );
				}
			);
		}
	)
}

//----------------------------------------------------------------------

if( !Promise.allSettled )
{
	Promise.allSettled = promises =>
		Promise.all(
			promises.map( ( promise, i ) =>
				promise
					.then( value => ( {
						status: "fulfilled",
						value,
					} ) )
					.catch( reason => ( {
						status: "rejected",
						reason,
					} ) )
			)
		);
}

//----------------------------------------------------------------------

class IPv4Addr
{
	constructor( ipv4str )
	{
		if( !validate_ip( ipv4str, 4 ) )
			throw Error( "invalid IPv4 address format" );

		this.ipv4str = ipv4str;
		this.ipv4num = new Uint32Array(1);
		this.to_number( ipv4str );
	}

	to_number()
	{
		let a = Uint8Array.from( this.ipv4str.split( "." ).reverse(), x => parseInt( x ) );
		//console.log( a );
		this.ipv4num[0]=a.reduce( ( acc, cur, index ) => { return ( cur << 8 * index ) + acc } );
	}

	as_number()
	{
		return this.ipv4num;
	}
}


class IPv4Subnet
{
	constructor( ipv4addr, prefix )
	{
		if( prefix < 1 || prefix > 30 )
			throw Error( "invalid prefix" );
		this.ipv4addr = ipv4addr;
		this.ipv4num = new Uint32Array(1);
		this.subnet_num = new Uint32Array(1);
		this.subnet_min_addr = new Uint32Array(1);
		this.subnet_max_addr = new Uint32Array(1);

		this.ipv4num=this.ipv4addr.as_number();

		this.subnet_num[0] = this.ipv4num[0] & ~( (1 << (32-prefix))-1 );
		this.subnet_min_addr[0] = this.subnet_num[0]+1;
		this.subnet_max_addr[0] = this.ipv4num[0] | ( (1 << (32-prefix))-1 );
		//console.log( this.subnet_max_addr[0].toString(2));
		this.subnet_max_addr[0] = this.subnet_max_addr[0] & 4294967294;

		//console.log( this.subnet_num[0].toString(2));
		//console.log( this.subnet_min_addr[0].toString(2));
		//console.log( this.subnet_max_addr[0].toString(2));
		this.prefix=prefix;
	}

	to_number()
	{
		let a = Uint8Array.from( this.ipv4str.split( "." ).reverse(), x => parseInt( x ) );
		//console.log( a );
		this.ipv4num[0]=a.reduce( ( acc, cur, index ) => { return ( cur << 8 * index ) + acc } );
	}

	get_subnet_str()
	{
		let res = new Uint32Array(4);
		res[0]=( this.subnet_num[0] >> 24 ) & 255 ;
		res[1]=( this.subnet_num[0] >> 16 ) & 255 ;
		res[2]=( this.subnet_num[0] >> 8 ) & 255 ;
		res[3]=( this.subnet_num[0] & 255 );
		return res.join(".")+"/"+this.prefix;
	}


	get_smallest_addr()
	{
		let res = new Uint32Array(4);
		res[0]=( this.subnet_min_addr[0] >> 24 ) & 255 ;
		res[1]=( this.subnet_min_addr[0] >> 16 ) & 255 ;
		res[2]=( this.subnet_min_addr[0] >> 8 ) & 255 ;
		res[3]=( this.subnet_min_addr[0] & 255 );
		return res.join(".");
	}

	get_biggest_addr()
	{
		let res = new Uint32Array(4);
		res[0]=( this.subnet_max_addr[0] >> 24 ) & 255 ;
		res[1]=( this.subnet_max_addr[0] >> 16 ) & 255 ;
		res[2]=( this.subnet_max_addr[0] >> 8 ) & 255 ;
		res[3]=( this.subnet_max_addr[0] & 255 );
		return res.join(".");
	}

}


